/******************************************************************************
 * get_raw_data.c
 * Federico Bareilles
 * 31/01/2003
 *
 * 24/07/2003: Add warning element in struct *tdata;
 *
 *****************************************************************************/

#define _POSIX_SOURCE 1         /* POSIX compliant source  */

#include <stdio.h>
#include <stdlib.h>
#include "tipper_red.h"


float get_date_bcc52(char *s, int sec_offset)
{
  float date = -1.0;

  int unit,h,m,sec,d_ref,sec_start;

  if(sscanf(s,"*%1d* %d %d %d %d %d",&unit,&h,&m,&sec,&d_ref,&sec_start) == 6){
    date=
      (float)d_ref+
      ((float)h + 
       ((float) m + 
	((float)sec+(float)sec_start+(float)sec_offset)/60.0)/ 60.0)/24.0;
  }
  /* PVERB("%f %d %d %d %d %d\n",date,h,m,sec,d_ref,sec_start); */
  return date;
}


int get_param(char *s,struct tdata *d)
{
  int res;
  d->n = 0;
  //PVERB("get_param: %s\n",s);
  res = sscanf(s,"%f %f %f %f %f %f %f %f",
	       &d->t_adq,&d->t_abs,&d->t_gun,&d->t_amp,
	       &d->v_comp,&d->v_comp_sigma,&d->v_gun,&d->v_gun_sigma);
  
  if(res != 8){
    d->error_flag |= BADRAWFILE;
  PVERB("get_param: Erro in raw data, match %d espect 8.\n%s\n",res,s);
  }
  return d->n;
}


int get_point(char *s, struct tpoint *p)
{
  int res;
  int n = 0;
  if((res=sscanf(s,"%d %f %f %f %f %f",
	    &p->pos,&p->airmass,&p->v_sky,&p->v_sky_sigma,
	    &p->v_ref,&p->v_ref_sigma)) == 6){
    n = 1;
  }else{
    if(res > 0){
      PVERB("get_point: Erro in raw data, match %d espect 6.\n%s\n",res,s);
    }
  }
  return n;
}
  

struct tdata *get_raw_data(FILE *fp_in, struct setup *s)
{
  struct tdata *d, *data;
  char line[160];
  int new_seq = 0;
  int sequence = -1;

  d = data = NULL;
  while( fgets(line, sizeof(line), fp_in) ){
    if(line[0] == '*' && line[2] == '*' ){
      new_seq = 1;
      sequence++;
      if( data == NULL){
	d = data = (struct tdata*) malloc(sizeof(struct tdata));
	data->next = NULL;
      }else{
	d = d->next = (struct tdata*) malloc(sizeof(struct tdata));
	d->next = NULL;
      }
      d->unit = line[1] - 0x30;
      d->error_flag = 0;
      d->date = get_date_bcc52(line, s->sec_offset);
      d->warning = NULL;
    }else{      
      if(new_seq == 1){
	if(get_param(line,d) == 0)
	  new_seq = 2;
      }else{
	if( new_seq == 2){
	  /* leer los puntos */
	  d->n += get_point(line,&d->points[d->n]);
	  if(d->n == MAX_ANG) 
	    new_seq = 0;
	}
      }
    }
    if(new_seq && line[0] == 'W'){
      if(d != NULL)  d->error_flag |= WARNING;
      line[strlen(line)-1] = 0;
      PVERB("# Date %8.3f(%3d): [%s]\n",d->date,sequence,line);

      if(d->error_flag & 1){
	d->warning = (char *) malloc(sizeof(char) * (strlen(line)-8));
	strcpy(d->warning,line+9);
      }
    }
  }
  PVERB("# Read %d points\n",sequence);
  s->data = data;

  return data;
}

int free_tdata(struct tdata *data)
{
  struct tdata *d;
  
  while(data != NULL){
    d=data->next;
    if(data->f) free(data->f);
    if(data->warning) free(data->warning);
    free(data);
    data = d;
  }
  return 0;
}

